--
--   { r=,  c= , keys={ "" ,  "" } },
local us_mapping = {
   -- Tab set
   { r=6 , c=0 , keys={ "RSHIFT" , "TAB" } },
   -- Tab clear
   { r=7 , c=0 , keys={ "RCONTROL" , "TAB" } },
   -- Step
   { r=7 , c=1 , keys={ "RCONTROL" , "PGDN" } },
   -- Del ln
   { r=4 , c=1 , keys={ "RSHIFT" , "DEL" } },
   -- Ins ln
   { r=3 , c=1 , keys={ "RSHIFT" , "INSERT" } },
   -- Prt all
   { r=0,  c=1 , keys={ "RCONTROL" , "HOME" } },
   -- Recall
   { r=6,  c=1 , keys={ "RSHIFT" ,  "PGUP" } },
   -- Auto st
   { r=1,  c=1 , keys={ "RCONTROL" ,  "END" } },
   -- Typwtr
   { r=5,  c=0 , keys={ "RCONTROL" ,  "INSERT" } },
   -- Clear
   { r=5,  c=6 , keys={ "RSHIFT" ,  "HOME" } },
   -- K0
   { r=7,  c=9 , keys={ "RCONTROL" ,  "1" } },
   -- K13
   { r=6,  c=4 , keys={ "RCONTROL" ,  "F10" } },
   -- K14
   { r=6,  c=3 , keys={ "RCONTROL" ,  "F11" } },
   -- K15
   { r=6,  c=2 , keys={ "RCONTROL" ,  "F12" } },
   -- Store
   { r=2,  c=9 , keys={ "RSHIFT" ,  "ENTER" } },
   -- Run
   { r=3,  c=7 , keys={ "RSHIFT" ,  "PGDN" } },
   -- Pause
   { r=2,  c=7 , keys={ "RSHIFT" ,  "ESC" } },
   -- Clear line
   { r=4,  c=6 , keys={ "RCONTROL" ,  "7PAD" , "OR" , "RSHIFT" , "END" } },
   -- Result
   { r=3,  c=6 , keys={ "RCONTROL" ,  "4PAD" } },
   -- Execute
   { r=0,  c=6 , keys={ "ENTERPAD" , "OR" , "RCONTROL" ,  "ENTER" } },
   -- Keypad E
   { r=4,  c=5 , keys={ "RCONTROL" ,  "SLASHPAD" } },
   -- Keypad (
   { r=4,  c=4 , keys={ "RCONTROL" ,  "8PAD" } },
   -- Keypad )
   { r=4,  c=3 , keys={ "RCONTROL" ,  "9PAD" } },
   -- Keypad ^
   { r=4,  c=2 , keys={ "RCONTROL" ,  "MINUSPAD" } },
   -- Keypad =
   { r=2,  c=6 , keys={ "RCONTROL" ,  "1PAD" } },
   -- Keypad ,
   { r=0,  c=3 , keys={ "RCONTROL" ,  "DELPAD" } },
}
local de_mapping = {
   { r=5 , c=9 ,  keys={ "RALT" , "8" , "OR" , "RALT" , "BACKSLASH2" } },
   { r=5 , c=8  , keys={ "RALT" , "9" , "OR" , "RALT" , "Q" } },
   { r=0 , c=15 , keys={ "LSHIFT" , "OR" , "RALT" , "BACKSLASH2" , "OR" , "RALT" , "Q" } }
}

local function dump_key_mapping (map , file)
      local n = map.r // 2
      local b = (map.r % 2) * 16 + map.c
      file:write("<port tag=\":KEY")
      file:write(tostring(n))
      file:write("\" type=\"KEYBOARD\" mask=\"")
      file:write(tostring(1 << b))
      file:write("\" defvalue=\"0\">\n")
      file:write("    <newseq type=\"standard\">\n")
      file:write("        ")
      local sep = ""
      for _ , k in ipairs(map.keys) do
         file:write(sep)
         if k == "OR" then
            file:write("OR")
         else
            file:write("KEYCODE_" .. k)
         end
         sep = " "
      end
      file:write("\n")
      file:write("    </newseq>\n")
      file:write("</port>\n")
end

local function dump_mapping (mapping , file)
   for _ , map in ipairs(mapping) do
      dump_key_mapping(map , file)
   end
end

local cb = function()
   local cfg_dir = manager:options().entries.cfg_directory:value()
   local romname = emu.romname()
   local cfg_file = romname .. ".cfg"

   local file = io.open(cfg_dir .. "/" .. cfg_file , "r")
   if file == nil then
      print("Configuration file" , cfg_file , "not found!")
      return
   end
   local tutto = file:read "*a"
   file:close()

   -- 6 is OPEN_FLAG_WRITE | OPEN_FLAG_CREATE
   file = emu.file(cfg_dir , 6)
   if file:open(cfg_file) then
      print("Cannot write to Configuration file" , cfg_file)
      return
   end
   
   local i , g
   local end1 , start2

   i , g = tutto:find("<input>" , 1)
   if i ~= nil then
      end1 = i - 1
      i , g = tutto:find("</input>%s*" , g + 1)
      start2 = g + 1
   else
      i , g = tutto:find("<system[^>]*>%s*" , 1)
      end1 = g
      start2 = g + 1
   end
   file:write(tutto:sub(1 , end1))
   file:write("<input>\n")
   dump_mapping(us_mapping , file)
   -- If machine has a German keyboard, apply additional mappings
   if romname:find("_de$") then
      dump_mapping(de_mapping , file)
   end
   file:write("</input>\n")
   file:write(tutto:sub(start2))

   print("Done!\n")
end

emu.register_stop(cb)
print("Callback registered..")
